const { MissingRequiredParamError, Type } = require('o-toolbox')
const BluePrintCommon = require('./BluePrintCommon')

// This UUID can be used during the render phase as a key for <Map> segments
let uuid = 0
function getGlobalUUID () {
  uuid += 1
  return uuid
}

class BluePrint {
  constructor (params = {}) {
    const { parent } = params
    MissingRequiredParamError.raiseIfMissing({ param: parent, paramName: 'parent' })

    this.uuid = getGlobalUUID()
    this.parent = Type.respondsTo(parent, 'asReference')
      ? parent.asReference()
      : parent

    this.publicCommands = [
      'command',
      'evaluate',
      'getCurrentUrl',
      'getRouter',
      'saveUrl',
      'setUrl',
      'gotoUrl',
      'invalidate',
      'route',
      'routeWithMethod',
      'respondsTo',
      'setVisibleContent',
      'getUUID'
    ]
    this.initialize(params)

    return this.asReference()
  }

  getClassName () {
    return this.constructor.name
  }

  getUUID () {
    return this.uuid
  }

  invalidate () {
    return this.parent.invalidate()
  }

  gotoUrl (url, state) {
    this.parent.gotoUrl(url, state)
  }

  saveUrl (url, state) {
    return this.parent.saveUrl(url, state)
  }

  setUrl (url) {
    return this.parent.setUrl(url)
  }

  getCurrentUrl () {
    return this.parent.getCurrentUrl()
  }

  evaluate ({ command, params }) {
    const acceptsCommand = this.acceptsCommand(command)
    if (acceptsCommand) { return Reflect.apply(this[command], this, params) }
    if (this.parent) { return this.parent.evaluate({ command, params }) }
    this.raiseCommandNotFoundError({ command, params })
  }

  respondsTo (command) {
    return this.acceptsCommand(command) || (this.parent && this.parent.respondsTo(command))
  }

  acceptsCommand (command) {
    return Type.respondsTo(this, command) && this.publicCommands.includes(command)
  }

  route () {
    return this.parent.getRouter().route(this)
  }

  routeWithMethod (methodName) {
    return this.parent.getRouter().routeWithMethod(methodName, this)
  }
}
BluePrintCommon.attachTo(BluePrint)

module.exports = BluePrint
