const { Type, AbsentMethodError } = require('o-toolbox')

class Router {
  /*
    Double dispatch the render to the proper component method
   */
  route (bluePrint) {
    const renderMethodName = this.getRenderMethodName(bluePrint)
    return this.routeWithMethod(renderMethodName, bluePrint)
  }

  /*
    Ask the bluePrint for the method instead of using the bluePrint class name.
    Class name is not always available at runtime.
   */
  getRenderMethodName (bluePrint) {
    if (!bluePrint.getRenderMethodName) {
      this.raiseMissingGetRenderMethodNameError(bluePrint)
    }
    const methodName = bluePrint.getRenderMethodName()
    if (!methodName) { this.raiseInvalidRenderMethodNameError(bluePrint) }
    return methodName
  }

  routeWithMethod (renderMethodName, bluePrint) {
    return Type.respondsTo(this, renderMethodName)
      ? Reflect.apply(this[renderMethodName], this, [bluePrint.asReference()])
      : this.raiseMissingRenderMethodError(this, renderMethodName)
  }

  raiseMissingRenderMethodError (bluePrintComponent, renderMethodName) {
    const message = `The class ${bluePrintComponent.constructor.name} must implement the method '${renderMethodName}(bluePrint)'`
    AbsentMethodError.raise({ message })
  }

  raiseMissingGetRenderMethodNameError (bluePrint) {
    const message = `The class ${bluePrint.constructor.name} must implement the method 'getRenderMethodName()'`
    AbsentMethodError.raise({ message })
  }

  raiseInvalidRenderMethodNameError (bluePrint) {
    const message = `The class ${bluePrint.constructor.name}.getRenderMethodName() method returned an empty string. Maybe it's missing the return statement?`
    AbsentMethodError.raise({ message })
  }
}

module.exports = Router
